@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2013 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU Lesser General Public License. See COPYING.LIB
@@ -49,11 +49,6 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 	struct flow_control *f = &p->flow;
 	int h, l;
 
-	if (fxt >= FX_SYNTH_0 && fxt <= FX_SYNTH_F) {
-		virt_seteffect(ctx, chn, fxt, fxp);
-		return;
-	}
-
 	switch (fxt) {
 	case FX_ARPEGGIO:
 		if (fxp != 0) {
@@ -214,7 +209,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 
 	case FX_VIBRATO:	/* Vibrato */
 		SET(VIBRATO);
-		SET_LFO_NOTZERO(&xc->vibrato, LSN(fxp) * 4, MSN(fxp));
+		SET_LFO_NOTZERO(&xc->vibrato, LSN(fxp) << 2, MSN(fxp));
 		break;
 	case FX_FINE_VIBRATO:	/* Fine vibrato (4x) */
 		SET(VIBRATO);
@@ -226,7 +221,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		} else {
 			RESET_PER(VIBRATO);
 		}
-		SET_LFO_NOTZERO(&xc->vibrato, LSN(fxp) * 4, MSN(fxp));
+		SET_LFO_NOTZERO(&xc->vibrato, LSN(fxp) << 2, MSN(fxp));
 		break;
 
 	case FX_TONE_VSLIDE:	/* Toneporta + vol slide */
@@ -375,7 +370,10 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			xc->gliss = fxp;
 			break;
 		case EX_VIBRATO_WF:	/* Set vibrato waveform */
-			set_lfo_waveform(&xc->vibrato, fxp & 3);
+			fxp &= 3;
+			if (HAS_QUIRK(QUIRK_S3MLFO) && fxp == 2)
+				fxp |= 0x10;
+			set_lfo_waveform(&xc->vibrato, fxp);
 			break;
 		case EX_FINETUNE:	/* Set finetune */
 			fxp <<= 4;
@@ -464,7 +462,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 	    fx_f_porta_up:
 		SET(FINE_BEND);
 		if (fxp)
-			xc->freq.fslide = -fxp * 4;
+			xc->freq.fslide = -fxp;
 		else if (xc->freq.slide > 0)
 			xc->freq.slide *= -1;
 		break;
@@ -472,7 +470,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 	    fx_f_porta_dn:
 		SET(FINE_BEND);
 		if (fxp)
-			xc->freq.fslide = fxp * 4;
+			xc->freq.fslide = fxp;
 		else if (xc->freq.slide < 0)
 			xc->freq.slide *= -1;
 		break;
@@ -609,10 +607,10 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		SET(FINE_BEND);
 		switch (MSN(fxp)) {
 		case 1:
-			xc->freq.fslide = -LSN(fxp);
+			xc->freq.fslide = -0.25 * LSN(fxp);
 			break;
 		case 2:
-			xc->freq.fslide = LSN(fxp);
+			xc->freq.fslide = 0.25 * LSN(fxp);
 			break;
 		}
 		break;
