@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2013 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU Lesser General Public License. See COPYING.LIB
@@ -11,6 +11,7 @@
 #include <fnmatch.h>
 #include "common.h"
 #include "synth.h"
+#include "loaders/loader.h"
 
 
 /*
@@ -124,6 +125,7 @@ void load_prologue(struct context_data *ctx)
 	m->quirk = 0;
 	m->read_event_type = READ_EVENT_MOD;
 	m->comment = NULL;
+	m->scan_cnt = NULL;
 
 	/* Set defaults */
     	m->mod.pat = 0;
@@ -206,6 +208,8 @@ int prepare_scan(struct context_data *ctx)
 	struct xmp_module *mod = &m->mod;
 	int i, ord;
 
+	if (mod->xxo == NULL || mod->xxp == NULL || mod->xxt == NULL)
+		return -XMP_ERROR_LOAD;
 	ord = 0;
 	while (ord < mod->len && mod->xxo[ord] >= mod->pat) {
 		ord++;
@@ -218,9 +222,17 @@ int prepare_scan(struct context_data *ctx)
 		return -XMP_ERROR_SYSTEM;
 
 	for (i = 0; i < mod->len; i++) {
-		int pat = mod->xxo[i];
-		m->scan_cnt[i] = calloc(1, pat >= mod->pat ?  1 :
-			mod->xxp[pat]->rows ? mod->xxp[pat]->rows : 1);
+		int pat_idx = mod->xxo[i];
+		struct xmp_pattern *pat;
+
+		/* Add pattern if referenced in orders */
+		if (pat_idx < mod->pat && !mod->xxp[pat_idx]) {
+			if (pattern_alloc(mod, pat_idx) < 0)
+				return -XMP_ERROR_SYSTEM;
+		}
+
+		pat = pat_idx >= mod->pat ? NULL : mod->xxp[pat_idx];
+		m->scan_cnt[i] = calloc(1, pat && pat->rows ? pat->rows : 1);
 		if (m->scan_cnt[i] == NULL)
 			return -XMP_ERROR_SYSTEM;
 	}
