@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2013 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU Lesser General Public License. See COPYING.LIB
@@ -17,7 +17,7 @@ static int asylum_test(HIO_HANDLE *, char *, const int);
 static int asylum_load(struct module_data *, HIO_HANDLE *, const int);
 
 const struct format_loader asylum_loader = {
-	"Asylum Music Format (AMF)",
+	"Asylum Music Format v1.0",
 	asylum_test,
 	asylum_load
 };
@@ -60,7 +60,7 @@ static int asylum_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	mod->smp = mod->ins;
 	mod->trk = mod->pat * mod->chn;
 
-	snprintf(mod->type, XMP_NAME_SIZE, "Asylum Music Format V1.0");
+	snprintf(mod->type, XMP_NAME_SIZE, "Asylum Music Format v1.0");
 
 	MODULE_INFO();
 
