@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2013 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU Lesser General Public License. See COPYING.LIB
@@ -26,7 +26,7 @@ static int med2_test(HIO_HANDLE *, char *, const int);
 static int med2_load (struct module_data *, HIO_HANDLE *, const int);
 
 const struct format_loader med2_loader = {
-	"MED 1.12 MED2 (MED)",
+	"MED 1.12 MED2",
 	med2_test,
 	med2_load
 };
