@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2013 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU Lesser General Public License. See COPYING.LIB
@@ -24,7 +24,7 @@ static int med4_test(HIO_HANDLE *, char *, const int);
 static int med4_load (struct module_data *, HIO_HANDLE *, const int);
 
 const struct format_loader med4_loader = {
-	"MED 2.10 MED4 (MED)",
+	"MED 2.10 MED4",
 	med4_test,
 	med4_load
 };
@@ -54,6 +54,7 @@ static const char *inst_type[] = {
 };
 #endif
 
+const unsigned MAX_CHANNELS = 16;
 
 static void fix_effect(struct xmp_event *event)
 {
@@ -98,34 +99,66 @@ static void fix_effect(struct xmp_event *event)
 	}
 }
 
-static inline uint8 read4(HIO_HANDLE *f, int *read4_ctl)
+struct stream {
+	HIO_HANDLE* f;
+	int has_nibble;
+	uint8 value;
+};
+
+static inline void stream_init(HIO_HANDLE* f, struct stream* s)
 {
-	static uint8 b = 0;
-	uint8 ret;
-
-	if (*read4_ctl & 0x01) {
-		ret = b & 0x0f;
-	} else {
-		b = hio_read8(f);
-		ret = b >> 4;
-	}
-
-	*read4_ctl ^= 0x01;
-
-	return ret;
+	s->f = f;
+	s->has_nibble = s->value = 0;
 }
 
-static inline uint16 read12b(HIO_HANDLE *f, int *read4_ctl)
+static inline unsigned stream_read4(struct stream* s)
 {
-	uint32 a, b, c;
+	s->has_nibble = !s->has_nibble;
+	if (!s->has_nibble) {
+	  return s->value & 0x0f;
+	} else {
+	  s->value = hio_read8(s->f);
+	  return s->value >> 4;
+	}
+}
 
-	a = read4(f, read4_ctl);
-	b = read4(f, read4_ctl);
-	c = read4(f, read4_ctl);
+static inline unsigned stream_read8(struct stream* s)
+{
+	unsigned a = stream_read4(s);
+	unsigned b = stream_read4(s);
+	return (a << 4) | b;
+}
 
+static inline unsigned stream_read12(struct stream* s)
+{
+	unsigned a = stream_read4(s);
+	unsigned b = stream_read4(s);
+	unsigned c = stream_read4(s);
 	return (a << 8) | (b << 4) | c;
 }
 
+static inline uint16 stream_read16(struct stream* s)
+{
+	unsigned a = stream_read4(s);
+	unsigned b = stream_read4(s);
+	unsigned c = stream_read4(s);
+	unsigned d = stream_read4(s);
+	return (a << 12) | (b << 8) | (c << 4) | d;
+}
+
+static inline uint16 stream_read_aligned16(struct stream* s, int bits)
+{
+	if (bits <= 4) {
+	  return stream_read4(s) << 12;
+	} else if (bits <= 8) {
+	  return stream_read8(s) << 8;
+	} else if (bits <= 12) {
+	  return stream_read12(s) << 4;
+	} else {
+	  return stream_read16(s);
+	}
+}
+
 struct temp_inst {
 	char name[32];
 	int loop_start;
@@ -134,13 +167,12 @@ struct temp_inst {
 	int transpose;
 };
 
-struct temp_inst temp_inst[32];
-
 static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 {
 	struct xmp_module *mod = &m->mod;
 	int i, j, k, y;
-	uint32 m0, mask;
+	uint8 m0;
+	uint64 mask;
 	int transp, masksz;
 	int pos, vermaj, vermin;
 	uint8 trkvol[16], buf[1024];
@@ -149,7 +181,7 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	int num_ins, num_smp;
 	int smp_idx;
 	int tempo;
-	int read4_ctl;
+	struct temp_inst temp_inst[64];
 	
 	LOAD_INIT();
 
@@ -164,9 +196,9 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	pos = hio_tell(f);
 	hio_seek(f, 0, SEEK_END);
 	if (hio_tell(f) > 2000) {
-		hio_seek(f, -1023, SEEK_CUR);
+		hio_seek(f, -1024, SEEK_CUR);
 		hio_read(buf, 1, 1024, f);
-		for (i = 0; i < 1012; i++) {
+		for (i = 0; i < 1013; i++) {
 			if (!memcmp(buf + i, "MEDV\000\000\000\004", 8)) {
 				vermaj = *(buf + i + 10);
 				vermin = *(buf + i + 11);
@@ -181,26 +213,25 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	m0 = hio_read8(f);
 
 	mask = masksz = 0;
-	for (i = 0; i < 8; i++, m0 <<= 1) {
+	for (i = 0; m0 != 0 && i < 8; i++, m0 <<= 1) {
 		if (m0 & 0x80) {
 			mask <<= 8;
 			mask |= hio_read8(f);
 			masksz++;
 		}
 	}
-	mask <<= (32 - masksz * 8);
+	mask <<= 8 * (sizeof(mask) - masksz);
 	/*printf("m0=%x mask=%x\n", m0, mask);*/
 
 	/* read instrument names in temporary space */
 
 	num_ins = 0;
-	for (i = 0; i < 32; i++, mask <<= 1) {
+	memset(&temp_inst, 0, sizeof(temp_inst));
+	for (i = 0; mask != 0 && i < 64; i++, mask <<= 1) {
 		uint8 c, size, buf[40];
 		uint16 loop_len = 0;
 
-		memset(&temp_inst[i], 0, sizeof (struct temp_inst));
-
-		if (~mask & 0x80000000)
+		if ((int64)mask > 0)
 			continue;
 
 		/* read flags */
@@ -225,7 +256,7 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			hio_read8(f);
 		if ((c & 0x08) == 0)	/* Tim Newsham's "span" */
 			hio_read8(f);
-		if ((c & 0x20) == 0)
+		if ((c & 0x30) == 0)
 			temp_inst[i].volume = hio_read8(f);
 		if ((c & 0x40) == 0)
 			temp_inst[i].transpose = hio_read8s(f);
@@ -242,6 +273,8 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 #ifdef MED4_DEBUG
 	printf("pat=%x len=%x\n", mod->pat, mod->len);
 #endif
+	if (mod->len > XMP_MAX_MOD_LENGTH)
+	  return -1;
 	hio_read(mod->xxo, 1, mod->len, f);
 
 	/* From MED V3.00 docs:
@@ -282,25 +315,43 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	D_(D_INFO "Play transpose: %d", transp);
 
-	for (i = 0; i < 32; i++)
+	for (i = 0; i < 64; i++)
 		temp_inst[i].transpose += transp;
 
-	hio_read8(f);
-	mod->chn = hio_read8(f);;
-	hio_seek(f, -2, SEEK_CUR);
+	/* Scan patterns to determine number of channels */
+	mod->chn = 0;
+	pos = hio_tell(f);
+
+	for (i = 0; i < mod->pat; i++) {
+		int size, plen, chn;
+
+		size = hio_read8(f);	/* pattern control block */
+		chn = hio_read8(f);
+		if (chn > mod->chn)
+			mod->chn = chn;
+		hio_read8(f);		/* skip number of rows */
+		plen = hio_read16b(f);
+
+		hio_seek(f, size + plen - 4, SEEK_CUR);
+	}
+
 	mod->trk = mod->chn * mod->pat;
 
 	if (pattern_init(mod) < 0)
 		return -1;
 
+	hio_seek(f, pos, SEEK_SET);
+
 	/* Load and convert patterns */
 	D_(D_INFO "Stored patterns: %d", mod->pat);
 
 	for (i = 0; i < mod->pat; i++) {
 		int size, plen, rows;
-		uint8 ctl[4], chmsk, chn;
+		uint8 ctl[4], chn;
+		unsigned chmsk;
 		uint32 linemask[8], fxmask[8], x;
 		int num_masks;
+		struct stream stream;
 
 #ifdef MED4_DEBUG
 		printf("\n===== PATTERN %d =====\n", i);
@@ -308,10 +359,12 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 #endif
 
 		size = hio_read8(f);	/* pattern control block */
+		pos = hio_tell(f);
 		chn = hio_read8(f);
+		if (chn > mod->chn)
+			return -1;
 		rows = (int)hio_read8(f) + 1;
 		plen = hio_read16b(f);
-
 #ifdef MED4_DEBUG
 		printf("size = %02x\n", size);
 		printf("chn  = %01x\n", chn);
@@ -359,14 +412,8 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			}
 		}
 
-
-		/* check block end */
-		if (hio_read8(f) != 0xff) {
-			D_(D_CRIT "error: module is corrupted");
-			return -1;
-		}
-
-		read4_ctl = 0;
+		hio_seek(f, pos + size, SEEK_SET);
+		stream_init(f, &stream);
 
 		for (y = 0; y < num_masks; y++) {
 
@@ -374,12 +421,12 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			int line = y * 32 + j;
 
 			if (linemask[y] & 0x80000000) {
-				chmsk = read4(f, &read4_ctl);
-				for (k = 0; k < 4; k++, chmsk <<= 1) {
+			  chmsk = stream_read_aligned16(&stream, chn);
+				for (k = 0; k < chn; k++, chmsk <<= 1) {
 					event = &EVENT(i, k, line);
 
-					if (chmsk & 0x08) {
-						x = read12b(f, &read4_ctl);
+					if (chmsk & 0x8000) {
+						x = stream_read12(&stream);
 						event->note = x >> 4;
 						if (event->note)
 							event->note += 48;
@@ -389,12 +436,12 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			}
 
 			if (fxmask[y] & 0x80000000) {
-				chmsk = read4(f, &read4_ctl);
-				for (k = 0; k < 4; k++, chmsk <<= 1) {
+			  chmsk = stream_read_aligned16(&stream, chn);
+				for (k = 0; k < chn; k++, chmsk <<= 1) {
 					event = &EVENT(i, k, line);
 
-					if (chmsk & 0x08) {
-						x = read12b(f, &read4_ctl);
+					if (chmsk & 0x8000) {
+						x = stream_read12(&stream);
 						event->fxt = x >> 8;
 						event->fxp = x & 0xff;
 						fix_effect(event);
@@ -421,9 +468,10 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		}
 
 		}
+		hio_seek(f, pos + size + plen, SEEK_SET);
 	}
 
-	mod->ins =  num_ins;
+	mod->ins = num_ins;
 
 	if (med_new_module_extras(m) != 0)
 		return -1;
@@ -432,13 +480,16 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	 * Load samples
 	 */
 	mask = hio_read32b(f);
-	hio_read16b(f);
+	if (mask == MAGIC4('M','E','D','V')) {
+	  mod->smp = 0;
 
-#ifdef MED4_DEBUG
-	printf("instrument mask: %08x\n", mask);
-#endif
-
-	hio_read16b(f);	/* ?!? */
+	  if (instrument_init(mod) < 0)
+		  return -1;
+	  hio_seek(f, -4, SEEK_CUR);
+	  goto parse_iff;
+	}
+	mask <<= 32;
+	mask |= hio_read32b(f);
 
 	mask <<= 1;	/* no instrument #0 */
 
@@ -446,16 +497,16 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	pos = hio_tell(f);
 	num_smp = 0;
 	{
-		int _len, _type, _mask = mask;
-		for (i = 0; i < 32; i++, _mask <<= 1) {
+		int _len, _type;
+		uint64 _mask = mask;
+		for (i = 0; _mask != 0 && i < 64; i++, _mask <<= 1) {
 			int _pos;
 
-			if (~_mask & 0x80000000)
+			if ((int64)_mask > 0)
 				continue;
-			hio_read16b(f);
-			_len = hio_read16b(f);
-			_type = (int16)hio_read16b(f);
 
+			_len = hio_read32b(f);
+			_type = (int16)hio_read16b(f);
 			_pos = hio_tell(f);
 
 			if (_type == 0 || _type == -2) {
@@ -478,20 +529,24 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	D_(D_INFO "Instruments: %d", mod->ins);
 
 	smp_idx = 0;
-	for (i = 0; i < num_ins; i++, mask <<= 1) {
-		int x1, length, type;
+	for (i = 0; mask != 0 && i < num_ins; i++, mask <<= 1) {
+		int length, type;
 		struct SynthInstr synth;
+		struct xmp_instrument *xxi;
+		struct xmp_subinstrument *sub;
+		struct xmp_sample *xxs;
 
-		if (~mask & 0x80000000)
+		if ((int64)mask > 0)
 			continue;
 
-		x1 = hio_read16b(f);
-		length = hio_read16b(f);
+		xxi = &mod->xxi[i];
+
+		length = hio_read32b(f);
 		type = (int16)hio_read16b(f);	/* instrument type */
 
-		strncpy((char *)mod->xxi[i].name, temp_inst[i].name, 32);
+		strncpy((char *)xxi->name, temp_inst[i].name, 32);
 
-		D_(D_INFO "\n[%2X] %-32.32s %d", i, mod->xxi[i].name, type);
+		D_(D_INFO "\n[%2X] %-32.32s %d", i, xxi->name, type);
 
 		/* This is very similar to MMD1 synth/hybrid instruments,
 		 * but just different enough to be reimplemented here.
@@ -519,33 +574,36 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			length = hio_read32b(f);
 			type = hio_read16b(f);
 
-			if (med_new_instrument_extras(&mod->xxi[i]) != 0)
+			if (med_new_instrument_extras(xxi) != 0)
 				return -1;
 
-			mod->xxi[i].nsm = 1;
+			xxi->nsm = 1;
 			if (subinstrument_alloc(mod, i, 1) < 0)
 				return -1;
 
-			MED_INSTRUMENT_EXTRAS(mod->xxi[i])->vts = synth.volspeed;
-			MED_INSTRUMENT_EXTRAS(mod->xxi[i])->wts = synth.wfspeed;
-			mod->xxi[i].sub[0].pan = 0x80;
-			mod->xxi[i].sub[0].vol = temp_inst[i].volume;
-			mod->xxi[i].sub[0].xpo = temp_inst[i].transpose;
-			mod->xxi[i].sub[0].sid = smp_idx;
-			mod->xxi[i].sub[0].fin = 0 /*exp_smp.finetune*/;
-			mod->xxs[smp_idx].len = length;
-			mod->xxs[smp_idx].lps = temp_inst[i].loop_start;
-			mod->xxs[smp_idx].lpe = temp_inst[i].loop_end;
-			mod->xxs[smp_idx].flg = temp_inst[i].loop_end > 1 ?
+			sub = &xxi->sub[0];
+
+			MED_INSTRUMENT_EXTRAS(*xxi)->vts = synth.volspeed;
+			MED_INSTRUMENT_EXTRAS(*xxi)->wts = synth.wfspeed;
+			sub->pan = 0x80;
+			sub->vol = temp_inst[i].volume;
+			sub->xpo = temp_inst[i].transpose;
+			sub->sid = smp_idx;
+			sub->fin = 0 /*exp_smp.finetune*/;
+
+			xxs = &mod->xxs[smp_idx];
+
+			xxs->len = length;
+			xxs->lps = temp_inst[i].loop_start;
+			xxs->lpe = temp_inst[i].loop_end;
+			xxs->flg = temp_inst[i].loop_end > 2 ?
 						XMP_SAMPLE_LOOP : 0;
 
 			D_(D_INFO "  %05x %05x %05x %02x %+03d",
-				       mod->xxs[smp_idx].len, mod->xxs[smp_idx].lps,
-				       mod->xxs[smp_idx].lpe, mod->xxi[i].sub[0].vol,
-				       mod->xxi[i].sub[0].xpo /*,
-				       mod->xxi[i].sub[0].fin >> 4*/);
+					xxs->len, xxs->lps, xxs->lpe,
+					sub->vol, sub->xpo /*, sub->fin >> 4*/);
 
-			if (load_sample(m, f, 0, &mod->xxs[smp_idx], NULL) < 0)
+			if (load_sample(m, f, 0, xxs, NULL) < 0)
 				return -1;
 
 			smp_idx++;
@@ -572,6 +630,10 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			synth.wforms = hio_read16b(f);
 			hio_read(synth.voltbl, 1, synth.voltbllen, f);;
 			hio_read(synth.wftbl, 1, synth.wftbllen, f);;
+			if (synth.wforms == 0xffff)	
+				continue;
+			if (synth.wforms > 64)
+				return -1;
 			for (j = 0; j < synth.wforms; j++)
 				synth.wf[j] = hio_read32b(f);
 
@@ -582,9 +644,6 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 					temp_inst[i].transpose /*,
 					exp_smp.finetune*/);
 
-			if (synth.wforms == 0xffff)	
-				continue;
-
 			if (med_new_instrument_extras(&mod->xxi[i]) != 0)
 				return -1;
 
@@ -592,27 +651,28 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			if (subinstrument_alloc(mod, i, synth.wforms) < 0)
 				return -1;
 
-			MED_INSTRUMENT_EXTRAS(mod->xxi[i])->vts = synth.volspeed;
-			MED_INSTRUMENT_EXTRAS(mod->xxi[i])->wts = synth.wfspeed;
+			MED_INSTRUMENT_EXTRAS(*xxi)->vts = synth.volspeed;
+			MED_INSTRUMENT_EXTRAS(*xxi)->wts = synth.wfspeed;
 
 			for (j = 0; j < synth.wforms; j++) {
-				mod->xxi[i].sub[j].pan = 0x80;
-				mod->xxi[i].sub[j].vol = temp_inst[i].volume;
-				mod->xxi[i].sub[j].xpo = temp_inst[i].transpose - 24;
-				mod->xxi[i].sub[j].sid = smp_idx;
-				mod->xxi[i].sub[j].fin = 0 /*exp_smp.finetune*/;
+				sub = &xxi->sub[j];
+
+				sub->pan = 0x80;
+				sub->vol = temp_inst[i].volume;
+				sub->xpo = temp_inst[i].transpose - 24;
+				sub->sid = smp_idx;
+				sub->fin = 0 /*exp_smp.finetune*/;
 
 				hio_seek(f, pos + synth.wf[j], SEEK_SET);
-/*printf("pos=%lx tell=%lx ", pos, hio_tell(f));*/
 
-				mod->xxs[smp_idx].len = hio_read16b(f) * 2;
-/*printf("idx=%x len=%x\n", synth.wf[j], mod->xxs[smp_idx].len);*/
-				mod->xxs[smp_idx].lps = 0;
-				mod->xxs[smp_idx].lpe = mod->xxs[smp_idx].len;
-				mod->xxs[smp_idx].flg = XMP_SAMPLE_LOOP;
+				xxs = &mod->xxs[smp_idx];
 
-				if (load_sample(m, f, 0, &mod->xxs[smp_idx],
-								NULL) < 0) {
+				xxs->len = hio_read16b(f) * 2;
+				xxs->lps = 0;
+				xxs->lpe = xxs->len;
+				xxs->flg = XMP_SAMPLE_LOOP;
+
+				if (load_sample(m, f, 0, xxs, NULL) < 0) {
 					return -1;
 				}
 
@@ -632,28 +692,30 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		}
 
                 /* instr type is sample */
-                mod->xxi[i].nsm = 1;
+                xxi->nsm = 1;
 		if (subinstrument_alloc(mod, i, 1) < 0)
 			return -1;
 		
-		mod->xxi[i].sub[0].vol = temp_inst[i].volume;
-		mod->xxi[i].sub[0].pan = 0x80;
-		mod->xxi[i].sub[0].xpo = temp_inst[i].transpose;
-		mod->xxi[i].sub[0].sid = smp_idx;
+		sub = &xxi->sub[0];
 
-		mod->xxs[smp_idx].len = length;
-		mod->xxs[smp_idx].lps = temp_inst[i].loop_start;
-		mod->xxs[smp_idx].lpe = temp_inst[i].loop_end;
-		mod->xxs[smp_idx].flg = temp_inst[i].loop_end > 1 ?
-						XMP_SAMPLE_LOOP : 0;
+		sub->vol = temp_inst[i].volume;
+		sub->pan = 0x80;
+		sub->xpo = temp_inst[i].transpose;
+		sub->sid = smp_idx;
+
+		xxs = &mod->xxs[smp_idx];
+
+		xxs->len = length;
+		xxs->lps = temp_inst[i].loop_start;
+		xxs->lpe = temp_inst[i].loop_end;
+		xxs->flg = temp_inst[i].loop_end > 2 ?  XMP_SAMPLE_LOOP : 0;
 
 		D_(D_INFO "  %04x %04x %04x %c V%02x %+03d",
-			mod->xxs[smp_idx].len, mod->xxs[smp_idx].lps,
-			mod->xxs[smp_idx].lpe,
-			mod->xxs[smp_idx].flg & XMP_SAMPLE_LOOP ? 'L' : ' ',
-			mod->xxi[i].sub[0].vol, mod->xxi[i].sub[0].xpo);
+				xxs->len, mod->xxs[smp_idx].lps, xxs->lpe,
+				xxs->flg & XMP_SAMPLE_LOOP ? 'L' : ' ',
+				sub->vol, sub->xpo);
 
-		if (load_sample(m, f, 0, &mod->xxs[smp_idx], NULL) < 0)
+		if (load_sample(m, f, 0, xxs, NULL) < 0)
 			return -1;
 
 		smp_idx++;
@@ -662,6 +724,7 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	hio_read16b(f);	/* unknown */
 
 	/* IFF-like section */
+parse_iff:
 	while (!hio_eof(f)) {
 		int32 id, size, s2, pos, ver;
 
