@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2013 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU Lesser General Public License. See COPYING.LIB
@@ -18,7 +18,7 @@ static int mmd1_test(HIO_HANDLE *, char *, const int);
 static int mmd1_load (struct module_data *, HIO_HANDLE *, const int);
 
 const struct format_loader mmd1_loader = {
-	"MED 2.10/OctaMED (MED)",
+	"MED 2.10/OctaMED",
 	mmd1_test,
 	mmd1_load
 };
@@ -66,13 +66,13 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	int ver = 0;
 	int smp_idx = 0;
 	uint8 e[4];
-	int song_offset;
-	int blockarr_offset;
-	int smplarr_offset;
-	int expdata_offset;
-	int expsmp_offset;
-	int songname_offset;
-	int iinfo_offset;
+	uint32 song_offset;
+	uint32 blockarr_offset;
+	uint32 smplarr_offset;
+	uint32 expdata_offset;
+	uint32 expsmp_offset;
+	uint32 songname_offset;
+	uint32 iinfo_offset;
 	int pos;
 	int bpm_on, bpmlen, med_8ch;
 
@@ -109,7 +109,8 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	 * song structure
 	 */
 	D_(D_WARN "load song");
-	hio_seek(f, start + song_offset, SEEK_SET);
+	if (hio_seek(f, start + song_offset, SEEK_SET) != 0)
+	  return -1;
 	for (i = 0; i < 63; i++) {
 		song.sample[i].rep = hio_read16b(f);
 		song.sample[i].replen = hio_read16b(f);
@@ -120,6 +121,8 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	}
 	song.numblocks = hio_read16b(f);
 	song.songlen = hio_read16b(f);
+	if (song.songlen > 256)
+	  return -1;
 	D_(D_INFO "song.songlen = %d", song.songlen);
 	for (i = 0; i < 256; i++)
 		song.playseq[i] = hio_read8(f);
@@ -143,22 +146,11 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	bpmlen = 1 + (song.flags2 & FLAG2_BMASK);
 	m->time_factor = MED_TIME_FACTOR;
 
-	/* From the OctaMEDv4 documentation:
-	 *
-	 * In 8-channel mode, you can control the playing speed more
-	 * accurately (to techies: by changing the size of the mix buffer).
-	 * This can be done with the left tempo gadget (values 1-10; the
-	 * lower, the faster). Values 11-240 are equivalent to 10.
-	 */
+	mmd_set_bpm(m, med_8ch, song.deftempo, bpm_on, bpmlen);
 
         mod->spd = song.tempo2;
-        mod->bpm = med_8ch ?
-			mmd_get_8ch_tempo(song.deftempo) :
-			(bpm_on ? song.deftempo * bpmlen / 16 : song.deftempo);
-
 	mod->pat = song.numblocks;
 	mod->ins = song.numsamples;
-	//mod->smp = mod->ins;
 	mod->len = song.songlen;
 	mod->rst = 0;
 	mod->chn = 0;
@@ -172,11 +164,13 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	for (i = 0; i < mod->ins; i++) {
 		uint32 smpl_offset;
 		int16 type;
-		hio_seek(f, start + smplarr_offset + i * 4, SEEK_SET);
+		if (hio_seek(f, start + smplarr_offset + i * 4, SEEK_SET) != 0)
+		  return -1;
 		smpl_offset = hio_read32b(f);
 		if (smpl_offset == 0)
 			continue;
-		hio_seek(f, start + smpl_offset, SEEK_SET);
+		if (hio_seek(f, start + smpl_offset, SEEK_SET) != 0)
+		  return -1;
 		hio_read32b(f);				/* length */
 		type = hio_read16b(f);
 		if (type == -1) {			/* type is synth? */
@@ -198,7 +192,8 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	expsmp_offset = 0;
 	iinfo_offset = 0;
 	if (expdata_offset) {
-		hio_seek(f, start + expdata_offset, SEEK_SET);
+		if (hio_seek(f, start + expdata_offset, SEEK_SET) != 0)
+		  return -1;
 		hio_read32b(f);
 		expsmp_offset = hio_read32b(f);
 		D_(D_INFO "expsmp_offset = 0x%08x", expsmp_offset);
@@ -234,12 +229,14 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	for (i = 0; i < mod->pat; i++) {
 		int block_offset;
 
-		hio_seek(f, start + blockarr_offset + i * 4, SEEK_SET);
+		if (hio_seek(f, start + blockarr_offset + i * 4, SEEK_SET) != 0)
+		  return -1;
 		block_offset = hio_read32b(f);
 		D_(D_INFO "block %d block_offset = 0x%08x", i, block_offset);
 		if (block_offset == 0)
 			continue;
-		hio_seek(f, start + block_offset, SEEK_SET);
+		if (hio_seek(f, start + block_offset, SEEK_SET) != 0)
+		  return -1;
 
 		if (ver > 0) {
 			block.numtracks = hio_read16b(f);
@@ -274,11 +271,13 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	for (i = 0; i < mod->pat; i++) {
 		int block_offset;
 
-		hio_seek(f, start + blockarr_offset + i * 4, SEEK_SET);
+		if (hio_seek(f, start + blockarr_offset + i * 4, SEEK_SET) != 0)
+		  return -1;
 		block_offset = hio_read32b(f);
 		if (block_offset == 0)
 			continue;
-		hio_seek(f, start + block_offset, SEEK_SET);
+		if (hio_seek(f, start + block_offset, SEEK_SET) != 0)
+		  return -1;
 
 		if (ver > 0) {
 			block.numtracks = hio_read16b(f);
@@ -366,8 +365,11 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	for (smp_idx = i = 0; i < mod->ins; i++) {
 		int smpl_offset;
 		char name[40] = "";
+		struct xmp_subinstrument *sub;
+		struct xmp_sample *xxs;
 
-		hio_seek(f, start + smplarr_offset + i * 4, SEEK_SET);
+		if (hio_seek(f, start + smplarr_offset + i * 4, SEEK_SET) != 0)
+		  return -1;
 		smpl_offset = hio_read32b(f);
 
 		D_(D_INFO "sample %d smpl_offset = 0x%08x", i, smpl_offset);
@@ -375,14 +377,16 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		if (smpl_offset == 0)
 			continue;
 
-		hio_seek(f, start + smpl_offset, SEEK_SET);
+		if (hio_seek(f, start + smpl_offset, SEEK_SET) != 0)
+		  return -1;
 		instr.length = hio_read32b(f);
 		instr.type = hio_read16b(f);
 
 		pos = hio_tell(f);
 
 		if (expdata_offset && i < expdata.i_ext_entries) {
-		    hio_seek(f, iinfo_offset + i * expdata.i_ext_entrsz, SEEK_SET);
+		    if (hio_seek(f, iinfo_offset + i * expdata.i_ext_entrsz, SEEK_SET) != 0)
+		      return -1;
 		    hio_read(name, 40, 1, f);
 		}
 
@@ -390,8 +394,8 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 		exp_smp.finetune = 0;
 		if (expdata_offset && i < expdata.s_ext_entries) {
-			hio_seek(f, expsmp_offset + i * expdata.s_ext_entrsz,
-							SEEK_SET);
+			if (hio_seek(f, expsmp_offset + i * expdata.s_ext_entrsz, SEEK_SET) != 0)
+			  return -1;
 			exp_smp.hold = hio_read8(f);
 			exp_smp.decay = hio_read8(f);
 			exp_smp.suppress_midi_off = hio_read8(f);
@@ -441,31 +445,31 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		if (subinstrument_alloc(mod, i, 1) < 0)
 			return -1;
 
-		mod->xxi[i].sub[0].vol = song.sample[i].svol;
-		mod->xxi[i].sub[0].pan = 0x80;
-		mod->xxi[i].sub[0].xpo = song.sample[i].strans;
-		mod->xxi[i].sub[0].sid = smp_idx;
-		mod->xxi[i].sub[0].fin = exp_smp.finetune << 4;
+		sub = &mod->xxi[i].sub[0];
 
-		mod->xxs[smp_idx].len = instr.length;
-		mod->xxs[smp_idx].lps = 2 * song.sample[i].rep;
-		mod->xxs[smp_idx].lpe = mod->xxs[smp_idx].lps + 2 *
-						song.sample[i].replen;
+		sub->vol = song.sample[i].svol;
+		sub->pan = 0x80;
+		sub->xpo = song.sample[i].strans;
+		sub->sid = smp_idx;
+		sub->fin = exp_smp.finetune << 4;
+
+		xxs = &mod->xxs[smp_idx];
+
+		xxs->len = instr.length;
+		xxs->lps = 2 * song.sample[i].rep;
+		xxs->lpe = xxs->lps + 2 * song.sample[i].replen;
+		xxs->flg = 0;
 
-		mod->xxs[smp_idx].flg = 0;
 		if (song.sample[i].replen > 1)
-			mod->xxs[smp_idx].flg |= XMP_SAMPLE_LOOP;
+			xxs->flg |= XMP_SAMPLE_LOOP;
 
 		D_(D_INFO "  %05x %05x %05x %02x %+3d %+1d",
-				mod->xxs[smp_idx].len,
-				mod->xxs[smp_idx].lps,
-				mod->xxs[smp_idx].lpe,
-				mod->xxi[i].sub[0].vol,
-				mod->xxi[i].sub[0].xpo,
-				mod->xxi[i].sub[0].fin >> 4);
+				xxs->len, xxs->lps, xxs->lpe,
+				sub->vol, sub->xpo, sub->fin >> 4);
 
-		hio_seek(f, start + smpl_offset + 6, SEEK_SET);
-		if (load_sample(m, f, 0, &mod->xxs[smp_idx], NULL) < 0)
+		if (hio_seek(f, start + smpl_offset + 6, SEEK_SET) != 0)
+		  return -1;
+		if (load_sample(m, f, 0, xxs, NULL) < 0)
 			return -1;
 
 		smp_idx++;
