@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2013 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU Lesser General Public License. See COPYING.LIB
@@ -70,7 +70,7 @@ static int s3m_test (HIO_HANDLE *, char *, const int);
 static int s3m_load (struct module_data *, HIO_HANDLE *, const int);
 
 const struct format_loader s3m_loader = {
-    "Scream Tracker 3 (S3M)",
+    "Scream Tracker 3",
     s3m_test,
     s3m_load
 };
@@ -202,27 +202,27 @@ static int s3m_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
     LOAD_INIT();
 
-    hio_read(&sfh.name, 28, 1, f);		/* Song name */
-    hio_read8(f);				/* 0x1a */
+    hio_read(&sfh.name, 28, 1, f);	/* Song name */
+    hio_read8(f);			/* 0x1a */
     sfh.type = hio_read8(f);		/* File type */
-    hio_read16l(f);				/* Reserved */
-    sfh.ordnum = hio_read16l(f);		/* Number of orders (must be even) */
-    sfh.insnum = hio_read16l(f);		/* Number of instruments */
-    sfh.patnum = hio_read16l(f);		/* Number of patterns */
+    hio_read16l(f);			/* Reserved */
+    sfh.ordnum = hio_read16l(f);	/* Number of orders (must be even) */
+    sfh.insnum = hio_read16l(f);	/* Number of instruments */
+    sfh.patnum = hio_read16l(f);	/* Number of patterns */
     sfh.flags = hio_read16l(f);		/* Flags */
-    sfh.version = hio_read16l(f);		/* Tracker ID and version */
+    sfh.version = hio_read16l(f);	/* Tracker ID and version */
     sfh.ffi = hio_read16l(f);		/* File format information */
     sfh.magic = hio_read32b(f);		/* 'SCRM' */
-    sfh.gv = hio_read8(f);			/* Global volume */
-    sfh.is = hio_read8(f);			/* Initial speed */
-    sfh.it = hio_read8(f);			/* Initial tempo */
-    sfh.mv = hio_read8(f);			/* Master volume */
-    sfh.uc = hio_read8(f);			/* Ultra click removal */
-    sfh.dp = hio_read8(f);			/* Default pan positions if 0xfc */
-    hio_read32l(f);				/* Reserved */
-    hio_read32l(f);				/* Reserved */
-    sfh.special = hio_read16l(f);		/* Ptr to special custom data */
-    hio_read(sfh.chset, 32, 1, f);		/* Channel settings */
+    sfh.gv = hio_read8(f);		/* Global volume */
+    sfh.is = hio_read8(f);		/* Initial speed */
+    sfh.it = hio_read8(f);		/* Initial tempo */
+    sfh.mv = hio_read8(f);		/* Master volume */
+    sfh.uc = hio_read8(f);		/* Ultra click removal */
+    sfh.dp = hio_read8(f);		/* Default pan positions if 0xfc */
+    hio_read32l(f);			/* Reserved */
+    hio_read32l(f);			/* Reserved */
+    sfh.special = hio_read16l(f);	/* Ptr to special custom data */
+    hio_read(sfh.chset, 32, 1, f);	/* Channel settings */
 
 #if 0
     if (sfh.magic != MAGIC_SCRM)
@@ -241,17 +241,11 @@ static int s3m_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
     copy_adjust(mod->name, sfh.name, 28);
 
-    /* Load and convert header */
-    mod->len = sfh.ordnum;
-    mod->ins = sfh.insnum;
-    mod->smp = mod->ins;
-    mod->pat = sfh.patnum;
-
-    pp_ins = calloc (2, mod->ins);
+    pp_ins = calloc (2, sfh.insnum);
     if (pp_ins == NULL)
 	goto err;
 
-    pp_pat = calloc (2, mod->pat);
+    pp_pat = calloc (2, sfh.patnum);
     if (pp_pat == NULL)
 	goto err2;
 
@@ -276,14 +270,34 @@ static int s3m_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		mod->xxc[i].pan = 0x80;
 	}
     }
+
+    if (sfh.ordnum <= XMP_MAX_MOD_LENGTH) {
+	mod->len = sfh.ordnum;
+	hio_read(mod->xxo, 1, mod->len, f);
+    } else {
+	mod->len = XMP_MAX_MOD_LENGTH;
+	hio_read(mod->xxo, 1, mod->len, f);
+	hio_seek(f, sfh.ordnum - XMP_MAX_MOD_LENGTH, SEEK_CUR);
+    }
+
+    mod->pat = -1;
+    for (i = 0; i < mod->len && mod->xxo[i] < 0xfe; ++i) {
+	if (mod->xxo[i] > mod->pat)
+            mod->pat = mod->xxo[i];
+    }
+    mod->pat++;
+    if (mod->pat > sfh.patnum)
+	mod->pat = sfh.patnum;
+
     mod->trk = mod->pat * mod->chn;
+    /* Load and convert header */
+    mod->ins = sfh.insnum;
+    mod->smp = mod->ins;
 
-    hio_read(mod->xxo, 1, mod->len, f);
-
-    for (i = 0; i < mod->ins; i++)
+    for (i = 0; i < sfh.insnum; i++)
 	pp_ins[i] = hio_read16l(f);
  
-    for (i = 0; i < mod->pat; i++)
+    for (i = 0; i < sfh.patnum; i++)
 	pp_pat[i] = hio_read16l(f);
 
     /* Default pan positions */
@@ -364,11 +378,7 @@ static int s3m_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	r = 0;
 	pat_len = hio_read16l(f) - 2;
 
-	/* Used to be (--pat_len >= 0). Replaced by Rudolf Cejka
-	 * <cejkar@dcse.fee.vutbr.cz>, fixes hunt.s3m
-	 * ftp://us.aminet.net/pub/aminet/mods/8voic/s3m_hunt.lha
-	 */
-	while (r < mod->xxp[i]->rows) {
+	while (pat_len >= 0 && r < mod->xxp[i]->rows) {
 	    b = hio_read8(f);
 
 	    if (b == S3M_EOR) {
@@ -433,15 +443,15 @@ static int s3m_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	    /* OPL2 FM instrument */
 
 	    hio_read(&sah.dosname, 12, 1, f);	/* DOS file name */
-	    hio_read(&sah.rsvd1, 3, 1, f);		/* 0x00 0x00 0x00 */
-	    hio_read(&sah.reg, 12, 1, f);		/* Adlib registers */
+	    hio_read(&sah.rsvd1, 3, 1, f);	/* 0x00 0x00 0x00 */
+	    hio_read(&sah.reg, 12, 1, f);	/* Adlib registers */
 	    sah.vol = hio_read8(f);
 	    sah.dsk = hio_read8(f);
 	    hio_read16l(f);
 	    sah.c2spd = hio_read16l(f);		/* C 4 speed */
 	    hio_read16l(f);
 	    hio_read(&sah.rsvd4, 12, 1, f);	/* Reserved */
-	    hio_read(&sah.name, 28, 1, f);		/* Instrument name */
+	    hio_read(&sah.name, 28, 1, f);	/* Instrument name */
 	    sah.magic = hio_read32b(f);		/* 'SCRI' */
 
 	    if (sah.magic != MAGIC_SCRI) {
@@ -466,23 +476,23 @@ static int s3m_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	    continue;
 	}
 
-	hio_read(&sih.dosname, 13, 1, f);		/* DOS file name */
+	hio_read(&sih.dosname, 13, 1, f);	/* DOS file name */
 	sih.memseg = hio_read16l(f);		/* Pointer to sample data */
 	sih.length = hio_read32l(f);		/* Length */
 	sih.loopbeg = hio_read32l(f);		/* Loop begin */
 	sih.loopend = hio_read32l(f);		/* Loop end */
 	sih.vol = hio_read8(f);			/* Volume */
-	sih.rsvd1 = hio_read8(f);			/* Reserved */
-	sih.pack = hio_read8(f);			/* Packing type (not used) */
+	sih.rsvd1 = hio_read8(f);		/* Reserved */
+	sih.pack = hio_read8(f);		/* Packing type (not used) */
 	sih.flags = hio_read8(f);		/* Loop/stereo/16bit samples flags */
-	sih.c2spd = hio_read16l(f);			/* C 4 speed */
-	sih.rsvd2 = hio_read16l(f);			/* Reserved */
+	sih.c2spd = hio_read16l(f);		/* C 4 speed */
+	sih.rsvd2 = hio_read16l(f);		/* Reserved */
 	hio_read(&sih.rsvd3, 4, 1, f);		/* Reserved */
 	sih.int_gp = hio_read16l(f);		/* Internal - GUS pointer */
 	sih.int_512 = hio_read16l(f);		/* Internal - SB pointer */
 	sih.int_last = hio_read32l(f);		/* Internal - SB index */
 	hio_read(&sih.name, 28, 1, f);		/* Instrument name */
-	sih.magic = hio_read32b(f);			/* 'SCRS' */
+	sih.magic = hio_read32b(f);		/* 'SCRS' */
 
 	if (x8 == 1 && sih.magic != MAGIC_SCRS) {
 	    D_(D_CRIT "error: instrument magic");
