@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2013 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU Lesser General Public License. See COPYING.LIB
@@ -28,7 +28,7 @@ static int xm_test (HIO_HANDLE *, char *, const int);
 static int xm_load (struct module_data *, HIO_HANDLE *, const int);
 
 const struct format_loader xm_loader = {
-    "Fast Tracker II (XM)",
+    "Fast Tracker II",
     xm_test,
     xm_load
 };
@@ -195,6 +195,22 @@ static int load_patterns(struct module_data *m, int version, HIO_HANDLE *f)
     return 0;
 }
 
+/* Sample number adjustment by Vitamin/CAIG */
+static struct xmp_sample* realloc_samples(struct xmp_sample* buf, int* size, int new_size)
+{
+  buf = realloc(buf, sizeof (struct xmp_sample) * new_size);
+  if (buf == NULL)
+    return NULL;
+  if (new_size > *size)
+    memset(buf + *size, 0, sizeof (struct xmp_sample) * (new_size - *size));
+  *size = new_size;
+  return buf;
+}
+
+//packed structures size
+#define XM_INST_HEADER_SIZE 33
+#define XM_INST_SIZE 208
+
 static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 {
     struct xmp_module *mod = &m->mod;
@@ -248,15 +264,16 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 	if (xxi->nsm) {
 	    if (subinstrument_alloc(mod, i, xxi->nsm) < 0)
 		return -1;
+	    if (xih.size < XM_INST_HEADER_SIZE)
+		return -1;
 
 	    /* for BoobieSqueezer (see http://boobie.rotfl.at/)
 	     * It works pretty much the same way as Impulse Tracker's sample
 	     * only mode, where it will strip off the instrument data.
 	     */
-	    if (xih.size == 0x26) {
-		hio_read8(f);
-		hio_read32l(f);
+      if (xih.size < XM_INST_HEADER_SIZE + XM_INST_SIZE) {
 		memset(&xi, 0, sizeof (struct xm_instrument));
+		hio_seek(f, xih.size - XM_INST_HEADER_SIZE, SEEK_CUR);
 	    } else {
 		hio_read(&xi.sample, 96, 1, f);	/* Sample map */
 		for (j = 0; j < 24; j++)
@@ -280,7 +297,7 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 		xi.v_fade = hio_read16l(f);	/* Volume fadeout */
 
 		/* Skip reserved space */
-		hio_seek(f, (int)xih.size - 33 /*sizeof (xih)*/ - 208 /*sizeof (xi)*/, SEEK_CUR);
+		hio_seek(f, (int)xih.size - (XM_INST_HEADER_SIZE + XM_INST_SIZE), SEEK_CUR);
 
 		/* Envelope */
 		xxi->rls = xi.v_fade;
@@ -297,12 +314,13 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 
 		if (xxi->aei.npt <= 0 || xxi->aei.npt > XMP_MAX_ENV_POINTS)
 		    xxi->aei.flg &= ~XMP_ENVELOPE_ON;
+		else
+		    memcpy(xxi->aei.data, xi.v_env, xxi->aei.npt * 4);
 
 		if (xxi->pei.npt <= 0 || xxi->pei.npt > XMP_MAX_ENV_POINTS)
 		    xxi->pei.flg &= ~XMP_ENVELOPE_ON;
-
-		memcpy(xxi->aei.data, xi.v_env, xxi->aei.npt * 4);
-		memcpy(xxi->pei.data, xi.p_env, xxi->pei.npt * 4);
+		else
+		    memcpy(xxi->pei.data, xi.p_env, xxi->pei.npt * 4);
 
 		for (j = 12; j < 108; j++) {
 		    xxi->map[j].ins = xi.sample[j - 12];
@@ -313,16 +331,23 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 
 	    for (j = 0; j < xxi->nsm; j++, sample_num++) {
 		struct xmp_subinstrument *sub = &xxi->sub[j];
-		struct xmp_sample *xxs = &mod->xxs[sample_num];
+		struct xmp_sample *xxs;
+
+		if (sample_num >= mod->smp) {
+		    mod->xxs = realloc_samples(mod->xxs, &mod->smp, mod->smp * 3 / 2);
+		    if (mod->xxs == NULL)
+			return -1;
+		}
+		xxs = &mod->xxs[sample_num];
 
 		xsh[j].length = hio_read32l(f);		/* Sample length */
 		xsh[j].loop_start = hio_read32l(f);	/* Sample loop start */
 		xsh[j].loop_length = hio_read32l(f);	/* Sample loop length */
-		xsh[j].volume = hio_read8(f);	/* Volume */
+		xsh[j].volume = hio_read8(f);		/* Volume */
 		xsh[j].finetune = hio_read8s(f);	/* Finetune (-128..+127) */
-		xsh[j].type = hio_read8(f);	/* Flags */
-		xsh[j].pan = hio_read8(f);	/* Panning (0-255) */
-		xsh[j].relnote = hio_read8s(f);	/* Relative note number */
+		xsh[j].type = hio_read8(f);		/* Flags */
+		xsh[j].pan = hio_read8(f);		/* Panning (0-255) */
+		xsh[j].relnote = hio_read8s(f);		/* Relative note number */
 		xsh[j].reserved = hio_read8(f);
 		hio_read(&xsh[j].name, 22, 1, f);	/* Sample_name */
 
@@ -335,8 +360,6 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 		sub->vra = xi.y_rate;
 		sub->vsw = xi.y_sweep;
 		sub->sid = sample_num;
-		if (sample_num >= MAX_SAMP)
-		    continue;
 
 		copy_adjust(xxs->name, xsh[j].name, 22);
 
@@ -360,8 +383,6 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 	    for (j = 0; j < xxi->nsm; j++) {
 		struct xmp_subinstrument *sub = &xxi->sub[j];
 
-		if (sample_num >= MAX_SAMP)
-		    continue;
 		D_(D_INFO " %1x: %06x%c%06x %06x %c V%02x F%+04d P%02x R%+03d",
 		    j, mod->xxs[sub->sid].len,
 		    mod->xxs[sub->sid].flg & XMP_SAMPLE_16BIT ? '+' : ' ',
@@ -380,15 +401,11 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 		}
 	    }
 	} else {
-	    /* The sample size is a field of struct xm_instrument_header that
-	     * should be in struct xm_instrument according to the (official)
-	     * format description. xmp puts the field in the header because
-	     * Fasttracker writes the modules this way. BUT there's some
-	     * other tracker or conversor out there following the specs
-	     * verbatim and thus creating modules incompatible with those
-	     * created by Fasttracker. The following piece of code is a
-	     * workaround for this problem (allowing xmp to play "Braintomb"
-	     * by Jazztiz/ART). (seek -4)
+	    /* Sample size should be in struct xm_instrument according to
+	     * the official format description, but FT2 actually puts it in
+	     * struct xm_instrument header. There's a tracker or converter
+	     * that follow the specs, so we must handle both cases (see 
+	     * "Braintomb" by Jazztiz/ART).
 	     */
 
 	    /* Umm, Cyke O'Path <cyker@heatwave.co.uk> sent me a couple of
@@ -399,11 +416,14 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 	     * generalization should take care of both cases.
 	     */
 
-	     hio_seek(f, (int)xih.size - 33 /*sizeof (xih)*/, SEEK_CUR);
+	     hio_seek(f, (int)xih.size - XM_INST_HEADER_SIZE, SEEK_CUR);
 	}
     }
-    mod->smp = sample_num;
-    mod->xxs = realloc(mod->xxs, sizeof (struct xmp_sample) * mod->smp);
+
+    /* Final sample number adjustment */
+    mod->xxs = realloc_samples(mod->xxs, &mod->smp, sample_num);
+    if (mod->xxs == NULL)
+	return -1;
 
     return 0;
 }
