@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2013 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU Lesser General Public License. See COPYING.LIB
@@ -288,8 +288,10 @@ static void process_volume(struct context_data *ctx, int chn, int t, int act)
 
 	finalvol = extras_get_volume(ctx, xc);
 
-	if (TEST(TREMOLO))
-		finalvol += get_lfo(&xc->tremolo) / 128;
+	if (TEST(TREMOLO)) {
+		finalvol += get_lfo(&xc->tremolo, 1 << 6);
+		update_lfo(&xc->tremolo);
+	}
 
 	CLAMP(finalvol, 0, m->volbase);
 
@@ -374,22 +376,36 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 		xc->noteslide.count--;
 	}
 
+	/* Instrument vibrato */
+	vibrato = get_lfo(&xc->insvib.lfo, (1024 * (1 + xc->insvib.sweep)));
+	update_lfo(&xc->insvib.lfo);
+	if (xc->insvib.sweep > 1) {
+		xc->insvib.sweep -= 2;
+	} else {
+		xc->insvib.sweep = 0;
+	}
+
 	/* Vibrato */
-
-	vibrato = get_lfo(&xc->insvib.lfo) / (1024 * (1 + xc->insvib.sweep));
-
 	if (TEST(VIBRATO) || TEST_PER(VIBRATO)) {
-		if (HAS_QUIRK(QUIRK_DEEPVIB)) {
-			vibrato += get_lfo(&xc->vibrato) >> 9;
+		int shift = HAS_QUIRK(QUIRK_VIBHALF) ? 10 : 9;
+		int vib = get_lfo(&xc->vibrato, 1 << shift);
+
+		if (HAS_QUIRK(QUIRK_VIBINV)) {
+			vibrato -= vib;
 		} else {
-			vibrato += get_lfo(&xc->vibrato) >> 10;
+			vibrato += vib;
+		}
+
+		if (t % p->speed != 0 || HAS_QUIRK(QUIRK_VIBALL)) {
+			update_lfo(&xc->vibrato);
 		}
 	}
 
-	period = xc->period + vibrato + extras_get_period(ctx, xc);
+	period = xc->period + extras_get_period(ctx, xc);
 
-	linear_bend = period_to_bend(period, xc->note, HAS_QUIRK(QUIRK_MODRNG),
-					xc->gliss, HAS_QUIRK(QUIRK_LINEAR));
+	linear_bend = period_to_bend(period + vibrato, xc->note,
+				HAS_QUIRK(QUIRK_MODRNG), xc->gliss,
+				HAS_QUIRK(QUIRK_LINEAR));
 
 	/* Envelope */
 
@@ -458,8 +474,10 @@ static void process_pan(struct context_data *ctx, int chn, int t, int act)
 	pan_envelope = get_envelope(&instrument->pei, xc->p_idx, 32, &end);
 	xc->p_idx = update_envelope(&instrument->pei, xc->p_idx, DOENV_RELEASE);
 
-	if (TEST(PANBRELLO))
-		panbrello = get_lfo(&xc->panbrello) / 512;
+	if (TEST(PANBRELLO)) {
+		panbrello = get_lfo(&xc->panbrello, 512);
+		update_lfo(&xc->panbrello);
+	}
 
 	finalpan = xc->pan.val + panbrello + (pan_envelope - 32) *
 				(128 - abs(xc->pan.val - 128)) / 32;
@@ -532,8 +550,6 @@ static void update_volume(struct context_data *ctx, int chn, int t)
 	CLAMP(xc->volume, 0, m->volbase);
 	CLAMP(p->gvol, 0, m->gvolbase);
 	CLAMP(xc->mastervol, 0, m->volbase);
-
-	update_lfo(&xc->tremolo);
 }
 
 static void update_frequency(struct context_data *ctx, int chn, int t)
@@ -543,8 +559,9 @@ static void update_frequency(struct context_data *ctx, int chn, int t)
 	struct channel_data *xc = &p->xc_data[chn];
 
 	if (t % p->speed != 0 || HAS_QUIRK(QUIRK_PBALL)) {
-		if (TEST(PITCHBEND) || TEST_PER(PITCHBEND))
+		if (TEST(PITCHBEND) || TEST_PER(PITCHBEND)) {
 			xc->period += xc->freq.slide;
+		}
 
 		/* Do tone portamento */
 		if (TEST(TONEPORTA) || TEST_PER(TONEPORTA)) {
@@ -571,7 +588,7 @@ static void update_frequency(struct context_data *ctx, int chn, int t)
 
 	if (t % p->speed == 0) {
 		if (TEST(FINE_BEND)) {
-			xc->period = (4 * xc->period + xc->freq.fslide) / 4;
+			xc->period += xc->freq.fslide;
 		}
 		if (TEST(FINE_NSLIDE)) {
 			xc->note += xc->noteslide.fslide;
@@ -596,19 +613,6 @@ static void update_frequency(struct context_data *ctx, int chn, int t)
 
 	xc->arpeggio.count++;
 	xc->arpeggio.count %= xc->arpeggio.size;
-
-	/* Update vibrato */
-	if (t % p->speed != 0 || HAS_QUIRK(QUIRK_VIBALL)) {
-		update_lfo(&xc->vibrato);
-
-		/* Update instrument vibrato */
-		update_lfo(&xc->insvib.lfo);
-		if (xc->insvib.sweep > 1) {
-			xc->insvib.sweep -= 2;
-		} else {
-			xc->insvib.sweep = 0;
-		}
-	}
 }
 
 static void update_pan(struct context_data *ctx, int chn, int t)
@@ -629,8 +633,6 @@ static void update_pan(struct context_data *ctx, int chn, int t)
 			xc->pan.val = 0xff;
 		}
 	}
-
-	update_lfo(&xc->panbrello);
 }
 
 static void play_channel(struct context_data *ctx, int chn, int t)
